@@ -17,13 +17,13 @@
 #include <global.h>
 
 constexpr auto ID_LEN = 10;
-constexpr auto DATABASE_REVISION = 5; // Keep MIGRATE_TO_LATEST_FROM in sync
+constexpr auto DATABASE_REVISION = 7; // Keep MIGRATE_TO_LATEST_FROM in sync
 #define MIGRATE_TO(n, current) \
     if (current < n) { \
         qDebug() << "Running migration" << #n; \
         migrationV##n(current); \
     }
-#define MIGRATE_TO_LATEST_FROM(current) MIGRATE_TO(5, current)
+#define MIGRATE_TO_LATEST_FROM(current) MIGRATE_TO(7, current)
 
 enum Column {
     IdColumn = 0,
@@ -42,7 +42,8 @@ enum Column {
     PendingDownloadColumn,
     ContentLocationColumn,
     ExpiresColumn,
-    SizeColumn
+    SizeColumn,
+    TapbacksColumn
 };
 
 Database::Database(QObject *parent)
@@ -64,12 +65,11 @@ Database::Database(QObject *parent)
     migrate();
 }
 
-QVector<Message> Database::messagesForNumber(const PhoneNumberList &phoneNumberList, const QString &id) const
+QVector<Message> Database::messagesForNumber(const PhoneNumberList &phoneNumberList, const QString &id, const bool last) const
 {
     QVector<Message> messages;
 
-    QSqlQuery fetch(m_database);
-    fetch.prepare(SL(R"(
+    QString sql = SL(R"(
         SELECT
             id,
             phoneNumber,
@@ -87,11 +87,18 @@ QVector<Message> Database::messagesForNumber(const PhoneNumberList &phoneNumberL
             pendingDownload,
             contentLocation,
             expires,
-            size
+            size,
+            tapbacks
         FROM Messages
         WHERE (phoneNumber == :phoneNumber AND :id IS NULL) OR id ==:id
         ORDER BY time DESC
-    )"));
+    )");
+    if (last) {
+        sql.append(SL(" LIMIT 1"));
+    }
+
+    QSqlQuery fetch(m_database);
+    fetch.prepare(sql);
     fetch.bindValue(SL(":phoneNumber"), phoneNumberList.toString());
     fetch.bindValue(SL(":id"), id);
     exec(fetch);
@@ -115,6 +122,7 @@ QVector<Message> Database::messagesForNumber(const PhoneNumberList &phoneNumberL
         message.contentLocation = fetch.value(Column::ContentLocationColumn).toString();
         message.expires = QDateTime::fromMSecsSinceEpoch(fetch.value(Column::ExpiresColumn).value<quint64>());
         message.size = fetch.value(Column::SizeColumn).toInt();
+        message.tapbacks = fetch.value(Column::TapbacksColumn).toString();
 
         messages.append(std::move(message));
     }
@@ -166,6 +174,38 @@ void Database::markMessageRead(const int id)
     exec(put);
 }
 
+void Database::updateMessageTapbacks(const QString &id, const QString tapbacks)
+{
+    QSqlQuery put(m_database);
+    put.prepare(SL("UPDATE Messages SET tapbacks = :tapbacks WHERE id == :id"));
+    put.bindValue(SL(":id"), id);
+    put.bindValue(SL(":tapbacks"), tapbacks);
+    exec(put);
+}
+
+QString Database::lastMessageWithText(const PhoneNumberList &phoneNumberList, const QString &text)
+{
+    QSqlQuery fetch(m_database);
+    fetch.prepare(SL("SELECT id FROM Messages WHERE phoneNumber == :phoneNumber AND text == :text ORDER BY time DESC LIMIT 1"));
+    fetch.bindValue(SL(":phoneNumber"), phoneNumberList.toString());
+    fetch.bindValue(SL(":text"), text);
+    exec(fetch);
+
+    fetch.first();
+    return fetch.value(0).toString();
+}
+
+QString Database::lastMessageWithAttachment(const PhoneNumberList &phoneNumberList)
+{
+    QSqlQuery fetch(m_database);
+    fetch.prepare(SL("SELECT id FROM Messages WHERE phoneNumber == :phoneNumber AND attachments IS NOT NULL ORDER BY time DESC LIMIT 1"));
+    fetch.bindValue(SL(":phoneNumber"), phoneNumberList.toString());
+    exec(fetch);
+
+    fetch.first();
+    return fetch.value(0).toString();
+}
+
 QVector<Chat> Database::chats() const
 {
     QVector<Chat> chats;
@@ -180,8 +220,15 @@ QVector<Chat> Database::chats() const
         Chat chat;
         chat.phoneNumberList = PhoneNumberList(fetch.value(0).toString());
         chat.unreadMessages = unreadMessagesForNumber(chat.phoneNumberList);
-        chat.lastMessage = lastMessageForNumber(chat.phoneNumberList);
-        chat.lastContacted = lastContactedForNumber(chat.phoneNumberList);
+
+        QVector<Message> messages = messagesForNumber(chat.phoneNumberList, QString(), true);
+
+        if (messages.length() > 0) {
+            chat.lastMessage = messages.at(0).text;
+            chat.lastContacted = messages.at(0).datetime;
+            chat.lastSentByMe = messages.at(0).sentByMe;
+            chat.lastAttachment = messages.at(0).attachments;
+        }
 
         chats.append(chat);
     }
@@ -203,28 +250,6 @@ int Database::unreadMessagesForNumber(const PhoneNumberList &phoneNumberList) co
     return fetch.value(0).toInt();
 }
 
-QString Database::lastMessageForNumber(const PhoneNumberList &phoneNumberList) const
-{
-    QSqlQuery fetch(m_database);
-    fetch.prepare(SL("SELECT text FROM Messages WHERE phoneNumber == :phoneNumber ORDER BY time DESC LIMIT 1"));
-    fetch.bindValue(SL(":phoneNumber"), phoneNumberList.toString());
-    fetch.exec();
-
-    fetch.first();
-    return fetch.value(0).toString();
-}
-
-QDateTime Database::lastContactedForNumber(const PhoneNumberList &phoneNumberList) const
-{
-    QSqlQuery fetch(m_database);
-    fetch.prepare(SL("SELECT time FROM Messages WHERE phoneNumber == :phoneNumber ORDER BY time DESC LIMIT 1"));
-    fetch.bindValue(SL(":phoneNumber"), phoneNumberList.toString());
-    exec(fetch);
-
-    fetch.first();
-    return QDateTime::fromMSecsSinceEpoch(fetch.value(0).toLongLong());
-}
-
 void Database::markChatAsRead(const PhoneNumberList &phoneNumberList)
 {
     QSqlQuery update(m_database);
@@ -461,3 +486,35 @@ void Database::migrationV5(uint current)
     fixDuplicateIds.prepare(SL("UPDATE Messages SET id = ROWID WHERE LENGTH(id) <> 10;"));
     Database::exec(fixDuplicateIds);
 }
+
+void Database::migrationV6(uint current)
+{
+    MIGRATE_TO(5, current);
+
+    QSqlQuery removeHtml(m_database);
+    removeHtml.prepare(SL("UPDATE Messages SET text = REPLACE(text,'&nbsp;', ' ')"));
+    Database::exec(removeHtml);
+    removeHtml.prepare(SL("UPDATE Messages SET text = REPLACE(text, '&lt;', '<')"));
+    Database::exec(removeHtml);
+    removeHtml.prepare(SL("UPDATE Messages SET text = REPLACE(text, '&gt;', '>')"));
+    Database::exec(removeHtml);
+    removeHtml.prepare(SL("UPDATE Messages SET text = REPLACE(text, '&quot;', '\"')"));
+    Database::exec(removeHtml);
+    removeHtml.prepare(SL("UPDATE Messages SET text = REPLACE(text, '&amp;', '&')"));
+    Database::exec(removeHtml);
+    removeHtml.prepare(SL("UPDATE Messages SET text = REPLACE(text, '<br />', CHAR(13))"));
+    Database::exec(removeHtml);
+    removeHtml.prepare(SL("UPDATE Messages SET text = REPLACE(text, '</a>', '')"));
+    Database::exec(removeHtml);
+    removeHtml.prepare(SL("UPDATE Messages SET text = REPLACE(text, SUBSTR(text, INSTR(text, '<a href='), INSTR(text, '>http') - INSTR(text, '<a href=') + CASE WHEN INSTR(text, '>http') > 0 THEN 1 ELSE 0 END), '')"));
+    Database::exec(removeHtml);
+}
+
+void Database::migrationV7(uint current)
+{
+    MIGRATE_TO(6, current);
+
+    QSqlQuery sql(m_database);
+    sql.prepare(SL("ALTER TABLE Messages ADD COLUMN tapbacks TEXT"));
+    Database::exec(sql);
+}
