@@ -5,16 +5,15 @@
 #pragma once
 
 #include <QObject>
+
 #include <databasethread.h>
 
-#include <qofonomanager.h>
+#include <mms.h>
+
+#include "daemoninterface.h"
 
 #include <optional>
 
-#include "messagemanager.h"
-
-class AsyncDatabase;
-
 class ChannelHandler : public QObject
 {
     Q_OBJECT
@@ -23,14 +22,15 @@ public:
     explicit ChannelHandler(std::optional<QString> &modemPath, QObject *parent = nullptr);
 
     AsyncDatabase &database();
-    MessageManager &msgManager();
+    org::kde::spacebar::Daemon *interface();
+    Mms &mms();
 
 private:
     DatabaseThread m_databaseThread;
-    MessageManager m_msgManager;
-    QOfonoManager m_manager;
+    org::kde::spacebar::Daemon *m_interface;
+    Mms m_mms;
 
-signals:
+Q_SIGNALS:
     void handlerReady();
     void channelOpen(const QString &phoneNumber);
 };
