@@ -9,6 +9,8 @@
 #include <QObject>
 #include <QSqlDatabase>
 
+#include <QCoro/Task>
+
 #include <contactphonenumbermapper.h>
 #include "database.h"
 #include "global.h"
@@ -17,13 +19,6 @@
 class MessageModel;
 class ChannelHandler;
 
-struct ChatData {
-    QString displayName;
-    QString phoneNumber;
-    int unreadMessages;
-    QDateTime lastContacted;
-};
-
 class ChatListModel : public QAbstractListModel
 {
     Q_OBJECT
@@ -37,6 +32,8 @@ public:
         UnreadMessagesRole,
         LastContactedRole,
         LastMessageRole,
+        LastSentByMeRole,
+        LastAttachmentRole,
         IsContactRole
     };
     Q_ENUM(Role)
@@ -51,6 +48,7 @@ public:
     Q_INVOKABLE void markChatAsRead(const PhoneNumberList &phoneNumberList);
     Q_INVOKABLE void restoreDefaults();
     Q_INVOKABLE void saveSettings();
+    Q_INVOKABLE QString attachmentsFolder(const PhoneNumberList &phoneNumberList) const;
 
     bool ready() const;
 
@@ -66,7 +64,7 @@ Q_SIGNALS:
     void chatsFetched();
 
 private:
-    void updateChats(const QVector<Chat> &chats);
+    QCoro::Task<void> fetchChatsInternal();
 
     ChannelHandler &m_handler;
     QVector<Chat> m_chats;
